"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

function getFilename(filename) {
  return filename && filename.replace(process.cwd(), '') || '';
}

function _default(babel) {
  var t = babel.types,
      template = babel.template;

  function getCatchBlockCode(asyncPath, awaitPath, plugins) {
    // get code location
    var loc = awaitPath.node.loc; // get function name

    var funcName = asyncPath.node.id.name || 'anonymous function'; // get filename

    var filename = getFilename(plugins.filename); // get error info

    var errorStatement = [template.statement('throw error;')()]; // opts选项: {console: true}

    if (plugins.opts && plugins.opts.console) {
      errorStatement.unshift(template.statement("console.error(\"auto catch at ".concat(filename, ".").concat(funcName, " at ").concat(loc.start.line, ",").concat(loc.start.column, "\", error);"))());
    } // insert catch block code


    return errorStatement;
  }

  return {
    name: 'elfin-auto-try-catch',
    visitor: {
      AwaitExpression: function AwaitExpression(path, plugins) {
        // 如果已经被try-catch包裹，过滤
        if (path.findParent(function (path) {
          return path.isTryStatement();
        })) return; // 判断是否在函数block作用域中

        var blockPath = path.findParent(function (path) {
          return path.isBlockStatement();
        });
        if (!blockPath) return; // 判断是否在 async函数中

        var asyncPath = blockPath.findParent(function (path) {
          return path.isFunctionDeclaration({
            async: true
          });
        });
        if (!asyncPath) return;
        var tryCatchAst = t.tryStatement(t.blockStatement(blockPath.node.body), t.catchClause(t.identifier('error'), t.blockStatement(getCatchBlockCode(asyncPath, path, plugins)))); // 最外层包裹try-catch

        blockPath.replaceWithMultiple([tryCatchAst]);
      }
    }
  };
}

module.exports = exports.default;